"""
Data structures that are to be used in a __packageInfo__ file.
"""

# used in l10n parameter to dist class
class Localization:
    """Defines a message catalog for a particular package"""
    def __init__(self, package, domain, sources=None, language=None):
        self.package = package
        self.domain = domain
        # Messages are extracted from the sources to generate .po files
        # Used by the 'generate_l10n' command
        self.sources = sources or []
        self.language = language or 'en_US'

# used in scripts parameter to dist class
class Script:
    def __init__(self, name, path):
        self.name = name
        index = path.rfind('.')
        if index == -1:
            # No package path
            self.module = ''
            self.identifier = path
        else:
            self.module = path[:index]
            self.identifier = path[index+1:]

    def __repr__(self):
        if self.module:
            path = self.module + '.' + self.identifier
        else:
            path = self.identifier
        return 'Script(%s, %s)' % (repr(self.name), repr(path))

class Document:
    """
    An XSLT rendered document. It creates both a XML, HTML and Text version of the document (if desired)
    """
    def __init__(self,
                 source,
                 outdir,
                 htmlStylesheets = None,
                 htmlOutfile = None,
                 textStylesheets=None,
                 textOutfile=None,
                 params=None,
                 title=None,
                 category='general'
                 ):
        self.source = source
        self.htmlStylesheets = htmlStylesheets
        self.textStylesheets = textStylesheets
        self.outdir = outdir
        self.htmlOutfile = htmlOutfile
        self.textOutfile = textOutfile
        self.params = params or {}
        self.title = title
        self.category = category

    def __repr__(self):
        return 'Document(%s, %s, %s, %s, %s, %s, %s, %s, %s)' % (repr(self.source),
                                                                 repr(self.outdir),
                                                                 repr(self.htmlStylesheets),
                                                                 repr(self.htmlOutfile),
                                                                 repr(self.textStylesheets),
                                                                 repr(self.textOutfile),
                                                                 repr(self.params),
                                                                 repr(self.title),
                                                                 repr(self.category))


