########################################################################
#
# File Name:            Create.Document.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CreateResource.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Create a new document in the 4Suite Server repository"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import ResourceTypes

forcedTypeMapping = {'xml':ResourceTypes.ResourceType.XML_DOCUMENT,
                     'xslt':ResourceTypes.ResourceType.XSLT_DOCUMENT,
                     'schematron':ResourceTypes.ResourceType.SCHEMATRON_DOCUMENT,
                     'rdf':ResourceTypes.ResourceType.RDF_DOCUMENT,
                     'container':ResourceTypes.ResourceType.CONTAINER,
                     'user':ResourceTypes.ResourceType.USER,
                     'xpathdocdef':ResourceTypes.ResourceType.XPATH_DOCUMENT_DEFINITION,
                     'xsltdocdef':ResourceTypes.ResourceType.XSLT_DOCUMENT_DEFINITION,
                     'group':ResourceTypes.ResourceType.GROUP,
                     'alias':ResourceTypes.ResourceType.ALIAS,
                     }


def Run(options,args):

    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    text = sourceStream.read()

    name = args['name']
    aliases = args.get('alias',[])

    try:

        commit = 0
        repo = CommandUtil.GetRepository(options,'4ss.create.document')

        docDef=options.get('docdef', None)

        try:
            doc = repo.createDocument(name,
                                      text,
                                      imt=options.get('imt', 'text/xml'),
                                      docDef=docDef,
                                      forcedType=forcedTypeMapping.get(options.get('type'))
                                      )
            for alias in aliases:
                doc.addAlias(alias)
            commit = 1

        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e))

    if args['sourceUri'] != '-':
        sourceStream.close()



def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          'Create a new XML document',
                          '--type=xml file:addrbook1.xml /addrboo1.xml',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the source from (The special value '-' means 'read from stdin')",
                                                                  str),
                                       Arguments.RequiredArgument('name',
                                                                  "The name of the new document",
                                                                  str),
                                       Arguments.ZeroOrMoreArgument('alias',
                                                                    "Additional aliases to assign to the new document.",
                                                                    str),
                                       ],
                          options = Options.Options([Options.Option('i',
                                                                    'imt=IMT',
                                                                    'The internet media type.  Defaults to text/xml.'),
                                                     Options.Option('D',
                                                                    'docdef=docdef',
                                                                    "The name of the document definition to use."),
                                                     Options.TypedOption('t',
                                                                         'type=',
                                                                         'The 4SS document type. If none is specified, then the type is inferred.',
                                                                         [('xml',"A XML Document"),
                                                                          ('xslt', "A XSLT Document"),
                                                                          ('schematron', "A Schematron Document"),
                                                                          ('rdf',"A RDF Document"),
                                                                          ('container',"A new Container"),
                                                                          ('user',"A new User"),
                                                                          ('xpathdocdef',"A new XPath Document Definition"),
                                                                          ('xsltdocdef',"A new XSLT Document Definition"),
                                                                          ('group',"A new group"),
                                                                          ('alias',"A new Alias")
                                                                          ]),
                                                     ]),

                          fileName = __file__,
                          )
    return cmd
