########################################################################
#
# File Name:            Fetch.Document.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Fetch a document from the 4SS repository"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil


def Run(options,args):
    output = sys.stdout
    quiet = options.get('quiet')
    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.document')
        try:
            doc = repo.fetchResource(
                args['path'],
                traverseAliases=(not options.has_key('no-traverse')))
            if not options.get('quiet', 0):
                sys.stdout.write("Document %s (%s)" % (doc.getPath(), doc.getImt()))
                sys.stdout.write("\n"+"-"*40+"\n")
            sys.stdout.write(doc.getContent())
            if not options.get('quiet', 0):
                sys.stdout.write("\n")
        finally:
            try:
                repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e))
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          'Fetch a document from the repository',
                          '/ftss/demos/index.xslt',
                          __doc__,
                          function=Run,
                          arguments=[Arguments.RequiredArgument('path',
                                                                  'document path to view',
                                                                  str),
                                       ],
                          options=Options.Options([Options.Option(None,
                                                                    'no-traverse',
                                                                    "Don't traverse aliases"),
                                                     Options.Option('q', 'quiet', 'Present no output except for errors or any resulting document'),
                                                     ]),
                          fileName=__file__,
                          )
    return cmd


