########################################################################
#
# File Name:            Fetch.File.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Fetch a Raw file from the 4SS Repository"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil


def Run(options,args):
    output = sys.stdout
    quiet = options.get('quiet')
    try:
        repo = CommandUtil.GetRepository(options,'4ss.fetch.rawfile')
        try:
            doc = repo.fetchResource(args['path'])

            if not quiet:
                sys.stderr.write("Raw File %s (%s)\n" % (args['path'],doc.getImt()))
                sys.stderr.write("-"*40+"\n")
            output.write(doc.getContent())

        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e))
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('rawfile',
                          'Fetch a file from the repository',
                          '/ftss/demos/4SSPowered.jpg > localfile.jpg',
                          __doc__,
                          function=Run,
                          arguments = [Arguments.RequiredArgument('path',
                                                                  'file path to view',
                                                                  str),
                                       ],
                          options=Options.Options([
                              Options.Option('q', 'quiet', 'Present no output except for errors or any resulting document'),
                              ]),
                          fileName=__file__,
                          )
    return cmd

