########################################################################
#
# File Name:            Fetch.User.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Fetch a user from the system.  This will display basic information about the user.  See fetch userdata for information about fetching more data from a user"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
from Ft.Server.Common import AclConstants, ResourceTypes

def Run(options, args):

    try:
        name = args['userName']
        repo = CommandUtil.GetRepository(options,'4ss.fetch.user')
        try:
            user = repo.fetchUserOrGroupByName(name)
            if user is not None:
                if user.isResourceType(ResourceTypes.ResourceType.USER):
                    print "User: %s" % user.getPath()
                    print "-"*10
                    print
                    print "Password Hash: %s" % user.getPassword()
                    print "Read ACL: %s" % user.getAcl(AclConstants.READ_ACCESS)
                    print "Write ACL: %s" % user.getAcl(AclConstants.WRITE_ACCESS)
                    print "Groups: [%s]" % ','.join(map(lambda x:x.getName(),user.getGroups()))
                else:
                    print "%s is a group, not a user." % name
            else:
                print "User %s does not exist." % name
        finally:
            repo.txRollback()
    except FtServerClientException, e:
        sys.stderr.write(str(e))


        return



def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('user',
                          'List information about a user in the repository',
                          'root',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('userName',
                                                                  'The name of the user for which to fetch info',
                                                                  str),
                                       ],

                          fileName = __file__
                          )
    return cmd
