########################################################################
#
# File Name:            Rdf.Add.py
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright 2001, Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Add RDF statements"""

import sys
from Ft.Lib import Uri
from Ft.Rdf.Statement import Statement
from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil
#from Ft.Server.Common.InstallLib import UnicodeFromXmlClobStream


def Run(options,args):
    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.rdf.add')
        try:
            scope = args['scope']
            doc = repo.fetchResource(scope)
            if options.get('document'):
                stream = Uri.BASIC_RESOLVER.resolve(options['document'])
                src = stream.read()
                doc.deserializeAndAdd(src)
            else:
                model = repo.getModel()
                s = Statement(args.get('subject'), args.get('predicate'),
                               args.get('object'), scope=scope
                               )
                model.add([s])
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        sys.stderr.write(str(e))
        return


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('add',
                          'Add RDF statements to a given scope',
                          '--document=spam.rdf /my/scope/path',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('scope',
                                                                  "The scope into which the statements are to be added",
                                                                  str),
                                       Arguments.OptionalArgument('subject',
                                                                  "The subject of the new statement",
                                                                  str),
                                       Arguments.OptionalArgument('predicate',
                                                                  "The predicate of the new statement",
                                                                  str),
                                       Arguments.OptionalArgument('object',
                                                                  "The object of the new statement",
                                                                  str)],
                          options = Options.Options([Options.Option('d', 'document=', 'Add statements deserialized from the given document'),
                                      ]),
                          fileName = __file__,
                          )
    return cmd

