########################################################################
#
# File Name:            Set.Acl.py
#
# Documentation:        http://docs.4suite.org/4SS/Lib/CommandLine.py.html
#
"""
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

__doc__ = """Set the access control information for an object in the system.  You need write access to the object to change its ACL values.."""

from Ft.Server.Client import FtServerClientException
from Ft.Server.Client.Commands import CommandUtil

from Ft.Server.Common import AclConstants

def Run(options,args):

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.set.acl')
        try:
            res = repo.fetchResource(args['path'], traverseAliases=0)
            done = 0
            while not done:
                #Display the current ACL
                print "Current ACL Settings for: %s" % res.getAbsolutePath()
                for name, key in (('Read', AclConstants.READ_ACCESS),
                                 ('Write', AclConstants.WRITE_ACCESS),
                                 ('Execute', AclConstants.EXECUTE_ACCESS),
                                 ('Delete', AclConstants.DELETE_ACCESS),
                                 ('Change Permissions', AclConstants.CHANGE_PERMISSIONS_ACCESS),
                                 ('Change Owner', AclConstants.CHANGE_OWNER_ACCESS),
                                 ):

                    print "%s ACL" % name
                    for ident,allowed in res.getAcl(key).items():
                        if ident[0] == ':': ident = ident[1:]
                        print "  %s --> %s" % (ident, allowed == AclConstants.ALLOWED and 'allowed' or 'denied')


                print "Please enter an action to perform on %s:" % res.getAbsolutePath()
                print "(s)et"
                print "(a)dd"
                print "(r)emove"
                print "(q)uit and save"
                print "(e)xit without save"
                while 1:
                    cmd = raw_input(">> ")
                    if not cmd or cmd.lower() not in ['s','a','r','q','e']:
                        print "Invalid command"
                    else:
                        cmd = cmd.lower()
                        break
                if cmd == 'e':
                    done = 1
                elif cmd == 'q':
                    commit = 1
                    done = 1
                else:
                    #Get information about what
                    phrase = 'set'
                    if cmd == 'a': phrase = 'add'
                    if cmd == 'r': phrase = 'remove'
                    print ""
                    print "Please enter the type of access: read | write | execute | delete | change owner | change permissions"
                    print ""
                    aclKey = raw_input("Please enter the type of access: ")
                    if not aclKey:
                        print "Invalid Access type"
                        continue

                    aclIdent = raw_input("Please enter the ident (i.e. username/groupname) to %s: " % phrase)
                    if not aclIdent:
                        print "Invalid Acl Ident"
                        continue

                    if cmd != 'r':
                        allowed = raw_input("Should %s be allowed %s on %s (yes/no): " % (aclIdent, aclKey, res.getPath()))
                        if not allowed:
                            print "Invalid Response"
                            continue
                        allowed = allowed.lower()
                        if allowed in ['y','yes']:
                            allowed = AclConstants.ALLOWED
                        elif allowed in ['n','no']:
                            allowed = AclConstants.DENIED
                        else:
                            print "Invalid Response: %s" % allowed
                            continue

                    if cmd == 's':
                        res.setAcl(aclKey, aclIdent, allowed)
                    elif cmd == 'r':
                        print aclKey
                        print aclIdent
                        res.removeAcl(aclKey, aclIdent)
                    elif cmd == 'a':
                        res.addAcl(aclKey, aclIdent, allowed)

        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass
    except FtServerClientException, e:
        raise


def Register():

    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('acl',
                          'Set Acl information about an object in the system.  You will be prompted further for the information to add/set/remove',
                          '/fourthought.com/index.xhtml',
                          __doc__,
                          function = Run,
                          arguments = [Arguments.RequiredArgument('path',
                                                                  "The path of an object in the system.",
                                                                  str),
                                       ],

                          fileName = __file__,
                          )
    return cmd




