########################################################################
#
# File Name:            Update.Document.py
#
#
"""
Update a resource int the repository
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""
__doc__ = """Update a document in the 4Suite Repository"""

import sys
from Ft.Server.Client import FtServerClientException
from Ft.Lib import Uri
from Ft.Server.Client.Commands import CommandUtil
#from Ft.Server.Common.InstallLib import UnicodeFromXmlClobStream

def Run(options, args):
    if args['sourceUri'] == '-':
        sourceStream = sys.stdin
    else:
        sourceStream = Uri.BASIC_RESOLVER.resolve(args['sourceUri'])

    #text = unicode(sourceStream.read())
    text = sourceStream.read()

    try:
        commit = 0
        repo = CommandUtil.GetRepository(options, '4ss.update.document')
        try:
            doc = repo.fetchResource(args['path'])
            doc.setContent(text)
            if options.has_key('imt'):
                doc.setImt(options['imt'])
            if options.has_key('docdef'):
                docDef = doc.fetchResource(options['docdef'])
                doc.setDocumentDefinition(docDef)
            if options.has_key('nodocdef'):
                doc.setDocumentDefinition(None)
            commit = 1
        finally:
            try:
                if commit:
                    repo.txCommit()
                else:
                    repo.txRollback()
            except:
                pass

    except FtServerClientException, e:
        sys.stderr.write(str(e))

    if args['sourceUri'] != '-':
        sourceStream.close()


def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('document',
                          """Update a document.""",
                          'localfile.xslt /fourthought.com/index.xslt',
                          __doc__,
                          function = Run,
                          arguments = [
                                       Arguments.RequiredArgument('sourceUri',
                                                                  "The URI to read the source from (Use '-' to read it from standard input)",
                                                                  str),
                                       Arguments.RequiredArgument('path',
                                                                  "The path of the existing document in the repository.",
                                                                  str),
                                       ],
                          options = Options.Options([Options.Option(None,
                                                                    'imt=','The internet media type to assign to the '\
                                                                    'document.  If not specified it is left unchanged.'),
                                                     Options.ExclusiveOptions([Options.Option(None,
                                                                                              'docdef=',
                                                                                              'The new document definition for this document'),
                                                                               Options.Option(None,
                                                                                              'nodocdef',
                                                                                              'remove the current doc def'),
                                                                               ]),
                                                     ]),
                          fileName = __file__,

                          )
    return cmd
