from Ft.Server import FtServerBaseException

import MessageSource

class FtServerClientException(FtServerBaseException):
    MessageSource = MessageSource

Error = MessageSource.Error

import os, sha, getpass

from Ft.Server.Common import ClAuthenticate
FTRPC_PORT = 8803

def GetAuthentication(options=None):
    """
    Get Authentication information for a Client connection to the repository
    1: look in the options dictionary for "anonymous"
    2: look in the options dictionary for "username"
    3: look in the options dictionary for "port"
    4: look in the options dictionary for "host"
    5:  If there is no username, then look for the FTSS_AGENT env var to get information
    6: If there still is no username, then look for FTSS_USERNAME env var
    7: If there still is no user name then prompt for it.
    8: If there is no password, then ask the password manager (from 4ss login)
    9: If there still is no password then prompt for it.
    10: If there is no host, then prompt for it
    11: If there is no port then promot for it
    """
    options = options or {}
    hasUserName = 0
    hasPasswd = 0
    userName = None
    passwdHash = None
    port = None
    host = None

    #First, check the options
    if options.has_key('anonymous'):
        hasUserName = 1
        hasPasswd = 1

    if not hasUserName and options.has_key('username'):
        userName = options['username']
        hasUserName = 1

    if not port and options.has_key('port'):
        port = int(options['port'])

    if not host and options.has_key('host'):
        host = options['host']

    #See if there is an agent
    if not hasUserName and os.environ.has_key('FTSS_AGENT'):
        u, p, h, po = eval(os.environ['FTSS_AGENT'])
        if not hasUserName:
            userName = u
            passwdHash = p
            hasUserName = 1
            hasPasswd = 1
        if not host:
            host = h
        if not port:
            port = int(po)
        
    #See if user name is in the environemt
    if not hasUserName:
        userName = os.environ.get('FTSS_USERNAME')
        if userName:
            hasUserName = 1

    #All else failed: prompt user
    if not hasUserName:
        userName = raw_input("4SS user name: ")
        hasUserName = 1

    if not hasPasswd and hasUserName:
        pm = ClAuthenticate.PasswdManager()
        pa, h, p = pm.getPassword(userName)
        if pa:
            passwdHash = pa
            hasPasswd = 1
        if not host and h:
            host = h
        if not port and p:
            port = int(p)
        
    if not hasPasswd and hasUserName:
        passwdHash = sha.new(getpass.getpass("Password for %s: " % userName)).hexdigest()
        hasPasswd = 1

    if not host:
        host = raw_input('Host <localhost>: ')
        host = host  or 'localhost'

    if not port:
        port = raw_input('Port <%i>: '%FTRPC_PORT)
        port = port or FTRPC_PORT
        port = int(port)

    return (userName,
            passwdHash,
            host,
            port)

def SmartLogin(options=None):
    userName,passwdHash,host,port = GetAuthentication(options)
    import Core
    repo = Core.GetRepository(userName,
                              passwdHash,
                              host,
                              port)

    return repo

