import urlparse, types
from Ft.Server.Common import Schema, ResourceTypes

from Ft.Server.Server import FtServerServerException, Error

def CreateInitialPath(documentRoot,
                      driver):
    """Called to get the first path in a transaction"""
    tempP = Path('/',driver,None,None,0)
    p =  tempP.normalize(documentRoot,0)
    #Now fix the display stuff
    p.displayPath = '/'
    p.displaySteps = ['/']
    p.displayPaths = ['/']
    p._documentRoot = p
    return p

def QuickCreate(path):
    p = CreateInitialPath('/',None)
    return p.normalize(path,0)

class Path:
    def __init__(self,pathString,driver,documentRoot,basePath,traverseLastAlias):


        self._driver = driver  #Needed to resolve aliases
        self._documentRoot = documentRoot #Needed to resolve absolute paths

        
        if basePath is None:
            #Very special case
            self.displayPath = '/'
            self.absolutePath = '/'
            self.displaySteps = ['/']
            self.absoluteSteps = ['/']
            self.displayPaths = ['/']
            self.absolutePaths = ['/']
            self.name = '/'
            self.suffix = ''
            self.resourceType = ResourceTypes.RESOURCE_CONTENT
        else:
            self._initPath(pathString,basePath,traverseLastAlias)
        
    def _initPath(self,pathString,basePath,traverseLastAlias):
        """Initialize ourselves from the basePath"""

        scheme, netloc, pathString, params, query, fragment = urlparse.urlparse(pathString)

        self.resourceType = ResourceTypes.RESOURCE_CONTENT
        suffix = params
        if params == 'metadata':
            self.resourceType = ResourceTypes.RESOURCE_METADATA
        elif params == 'cache':
            self.resourceType = ResourceTypes.RESOURCE_CACHE

        if pathString[0] == '/':
            #We got a whole
            raise "Should not have a '/'"
        
        #When we get in here, we have a base path and the new path is based off of the
        #base path.  Always
        #Do this based on steps because it is the easiest
        self.suffix = suffix

        dSteps = basePath.displaySteps[:]
        dPaths = basePath.displayPaths[:]
        aPaths = basePath.absolutePaths[:]
        
        fields = filter(None,pathString.split('/'))

        for curStep in fields:
            lastStep = curStep is fields[-1]
            if curStep == '.':
                continue
            elif curStep == '..':
                #The limiting factor here is the dPath
                if len(dPaths) > 1:
                    dSteps = dSteps[:-1]
                    dPaths = dPaths[:-1]
                    aPaths = aPaths[:-1]
                continue

            dSteps.append(curStep)
            if dPaths[-1] == '/':
                dPaths.append('/' + curStep)
            else:
                dPaths.append(dPaths[-1] + '/' + curStep)

            #See if this step is an alias.  If it is, then we need to
            #Adjust the absolute step and path accordingly

            if aPaths[-1] == '/':
                testPath = '/' + curStep
            else:
                testPath = aPaths[-1] + '/' + curStep

            
            if self._driver and (not lastStep or traverseLastAlias):
                while len(self._driver.getSystemModel().complete(testPath,Schema.TYPE,Schema.g_rdfResourceTypes[ResourceTypes.ResourceType.ALIAS])):
                    #It is!!!
                    #Actually add the alias reference to the path
                    testPath = self._driver.getSystemModel().complete(testPath,
                                                                      Schema.ALIAS_REFERENCE,
                                                                      None)[0].object
            aPaths.append(testPath)
                
        self.displayPath = dPaths[-1]
        if suffix:
            self.displayPath = self.displayPath + ';' + suffix
        self.absolutePath = aPaths[-1]
        self.displaySteps = dSteps
        self.name = dSteps[-1]
        self.displayPaths = dPaths
        self.absolutePaths = aPaths 


    def normalize(self,pathString,traverseLastAlias):
        """Create a new path instance using us as the base"""

        if type(pathString) not in [types.StringType,
                                    types.UnicodeType] or not pathString:
            raise FtServerServerException(Error.UNKNOWN_PATH,path=pathString)

        #Quick normalization of // at the beginning
        while pathString[:2] == '//':
            pathString = pathString[1:]

        scheme, netloc, pathString, params, query, fragment = urlparse.urlparse(pathString)

        rt = ResourceTypes.RESOURCE_CONTENT
        suffix = params
        if params == 'metadata':
            rt = ResourceTypes.RESOURCE_METADATA
        elif params == 'cache':
            rt = ResourceTypes.RESOURCE_CACHE

        if suffix:
            sufPathString = pathString + ';' + suffix
        else:
            sufPathString = pathString

        if pathString[0] == '/':
            #Should traverse aliase logic fall in here?
            if len(pathString) == 1:
                if self._documentRoot:
                    dr = self._documentRoot
                else:
                    #If we have no document Root then we are the document Root
                    dr = self
                if rt == dr.resourceType and dr.suffix == suffix:
                    return dr
                if suffix:
                    return dr.normalize('.;'+suffix,0)
                return dr.normalize('.;content',0)
            if self._documentRoot:
                return self._documentRoot.normalize(sufPathString[1:],traverseLastAlias)
            #Again, if we have no document root, then we are the document Root
            return self.normalize(sufPathString[1:],traverseLastAlias)

        return Path(sufPathString,
                    self._driver,
                    self._documentRoot,
                    self,
                    traverseLastAlias)
        
    def getParentPath(self):
        return self.normalize('..',0)


    def __str__(self):
        return "<Path '%s' abs='%s'>" % (self.displayPath,
                                         self.absolutePath)




