#import os, cPickle, sys, time
import urllib
import XmlDocumentImp

from Ft.Xml.Xslt import StylesheetReader
from Ft.Server.Common import ResourceTypes, Schema, XmlLib
from Ft.Server import FTSERVER_NAMESPACE
from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Server.Drivers import FtssInputSource
from Ft.Xml import Xslt
class XsltDocumentImp(XmlDocumentImp.XmlDocumentImp):
    """
    An Xslt Document
    """
    resourceType = ResourceTypes.ResourceType.XSLT_DOCUMENT

    def asStylesheet(self):
        """
        Returns this resource as an Xslt stylesheet
        """
        self._verifyTx()
        #See if anything was changed
        st = None
        if self._hasModifiedDependencies(self._path.absolutePath):
            #Force a recompile
            try:
                self.setContent(self.getContent())
            except FtServerServerException, e:
                if e.code == Error.PERMISSION_DENIED:
                    print "Unable to save recompiled document"

                    functions,extElements = self.getExtFunctionsAndElements()

                    isrc = FtssInputSource.FtssInputSourceFactory.fromString(self.getContent(),
                                                                             self._path.absolutePath,
                                                                             self._driver)
                    reader = StylesheetReader.StylesheetReader(extElements = extElements)
                    st = reader.fromSrc(isrc)
                    pass
                else:
                    raise
        if not st:
            r = StylesheetReader.StylesheetReader()
            st = r.fromInstant(self._driver.fetchResourceCache(self._path),
                               self.getAbsolutePath())
        return st

    def setContent(self, src):
        XmlDocumentImp.XmlDocumentImp.setContent(self, src)

        extFunctions,extElements = self.getExtFunctionsAndElements()

        isrc = FtssInputSource.FtssInputSourceFactory.fromString(src,
                                                                 self._path.absolutePath,
                                                                 self._driver)
        
        reader = StylesheetReader.StylesheetReader(extElements = extElements)
        sty = reader.fromSrc(isrc)

        #Compile the stylesheet
        self._driver.setResourceCache(self._path,
                                      sty.root
                                      )

    def setDocumentDefinition(self, newDocDef):
        """
        Reset our document definition and regenerate our self
        """
        XmlDocumentImp.XmlDocumentImp.setDocumentDefinition(self,newDocDef)
        #In case there was a doc def change
        self.setContent(self.getContent())

    def getExtFunctionsAndElements(self):
        """
        All resource get to use the builtin functions
        """
        functions,elements = XmlDocumentImp.XmlDocumentImp.getExtFunctionsAndElements(self)
        dd = self.getDocumentDefinition()
        if dd:
            cp = dd.getCreationParams()
            if cp:
                f,e = cp.getExtFunctionsAndElements()

                functions.update(f)
                elements.update(e)
        return functions,elements
                


    def _hasModifiedDependencies(self, path):

        model = self._driver.getSystemModel()
        curLmd = model.complete(path,Schema.MODIFIED_DATE, None)[0].object

        for stmt in model.complete(path, Schema.STYLESHEET_DEPENDENCY,None):
            testPath = urllib.basejoin(path, stmt.object)
            test = model.complete(testPath, Schema.MODIFIED_DATE,None)
            if test and test[0].object > curLmd:
                return 1
            if self._hasModifiedDependencies(testPath): return 1
        return 0

