########################################################################
#
# File Name:            Group.py
#
"""
XSLT and XPath extensions supporting the 4SS group API
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml.XPath import Conversions
from Ft.Xml.Xslt import XsltElement, AttributeInfo, ContentInfo

from Ns import SCORE_NS
import FtssXsltBase

def GetMemberPaths(context, group):
    """
    Returns the full paths to all user resources that are members
    of the specified group
    """
    repo = context.processor._repository    
    res= repo.fetchResource(Conversions.StringValue(group))
    
    
    proc = context.processor
    proc.pushResult()
    for path in res.getMemberPaths():        
        proc.writers[-1].startElement('path')
        proc.writers[-1].text(path)
        proc.writers[-1].endElement('path')

    frag = proc.popResult()
    context.rtfs.append(frag)
    stmt_nodeset = frag.childNodes
    return stmt_nodeset


class AddMemberElement(XsltElement):
    """
    Adds a user or group to the specified group
    """

    content = ContentInfo.Empty

    legalAttrs = {
        'path' : AttributeInfo.UriReferenceAvt(required=1,
                                              description='The path of the group '
                                              'to which to add this member.'),
        'member-path' : AttributeInfo.UriReferenceAvt(required=1,
                                                     description='The path of the '
                                                     'new member (user or group) to '
                                                     'add to this group.'),
        }

    childArgument = None

    def instantiate(self, context, processor):
        context.setProcessState(self)
        repo = context.processor._repository
        
        path = self._path.evaluate(context)
        memberPath = self._member_path.evaluate(context)        
        
        base = FtssXsltBase.FetchBaseObject(processor, path)
        newMember = repo.fetchResource(memberPath)
        base.addMember(newMember)

        return (context,)


class RemoveMemberElement(XsltElement):
    """
    Removes a user or group from the specified group
    """

    content = ContentInfo.Empty

    legalAttrs = {
        'path' : AttributeInfo.UriReferenceAvt(required=1,
                                              description='The path of the group '
                                              'to which to remove this member.'),
        'member-path' : AttributeInfo.UriReferenceAvt(required=1,
                                                     description='The path of the '
                                                     'member (user or group) to '
                                                     'remove from this group.'),
        }

    childArgument = None

    def instantiate(self, context, processor):
        context.setProcessState(self)

        repo = context.processor._repository
        
        path = self._path.evaluate(context)
        memberPath = self._member_path.evaluate(context)        
        
        base = FtssXsltBase.FetchBaseObject(processor, path)
        oldMember = repo.fetchResource(memberPath)
        base.removeMember(oldMember)

        return (context,)


ExtFunctions = {
    (SCORE_NS, 'get-member-paths'): GetMemberPaths,    
}

ExtElements = {
    (SCORE_NS, 'add-member'): AddMemberElement,
    (SCORE_NS, 'remove-member'): RemoveMemberElement,
}

