########################################################################
#
# File Name:            XsltExt.py
#
# Documentation:        http://docs.4suite.org/4SS/Protocols/Http/XsltExt.py.html
#
"""
XSLT and XPath extensions for use to access the 4SS API
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Server import RESERVED_NAMESPACE
from Ft.Xml.XPath import Conversions
from Ft.Xml.Xslt import XsltElement, ContentInfo, AttributeInfo
import types
from Ns import HTTP_NS

def LocalPath(context, path):
    path = Conversions.StringValue(path)
    handler = context.processor.extensionParams[(RESERVED_NAMESPACE, 'handler')]

    docRoot = handler.documentRoot
    if docRoot and path[:len(docRoot)] == docRoot:
        path = path[len(docRoot):]
    return path


def GetQueryArguments(context):    
    queryArgs = context.processor.extensionParams[(RESERVED_NAMESPACE,'queryArgs')]        
    proc = context.processor
    proc.pushResult()    
    proc.writers[-1].startElement('requestParameters')    
    
    for name, values in queryArgs.items():
        #print "(%s,%s)"%(name,repr(values))                          
        for value in values:            
            proc.writers[-1].startElement(name)
            proc.writers[-1].text(value)
            proc.writers[-1].endElement(name)
    proc.writers[-1].endElement('requestParemeters')
        
    frag = proc.popResult()    
    context.rtfs.append(frag)
    stmt_nodeset = frag.childNodes    
    return stmt_nodeset       
    

class ResponseUriElement(XsltElement):

    content = ContentInfo.Empty
    legalAttrs = {
        'uri' : AttributeInfo.UriReferenceAvt(required=1,
                                              description='The uri to use for '
                                              'the response redirect.'),
        }

    childArgument = None

    def instantiate(self, context, processor):
        context.setProcessState(self)
        uri = self._uri.evaluate(context)
        context.processor.extensionParams[(RESERVED_NAMESPACE, 'response-uri')] = uri
        return (context,)


class RequestHeadersElement(XsltElement):

    content = ContentInfo.Empty
    legalAttrs = {}

    def instantiate(self, context, processor):
        handler = context.processor.extensionParams.get((RESERVED_NAMESPACE,
                                                         'handler'))
        for k, v in handler.headers_in.items():
            processor.writers[-1].startElement('Header')
            processor.writers[-1].startElement('Name')
            processor.writers[-1].text(k)
            processor.writers[-1].endElement('Name')
            if type(v) != types.ListType:
                v = [v]
            for actual in v:
                processor.writers[-1].startElement('Value')
                processor.writers[-1].text(actual)
                processor.writers[-1].endElement('Value')
            processor.writers[-1].endElement('Header')
        return (context,)


ExtElements = {
    (HTTP_NS, 'response-uri'): ResponseUriElement,
    (HTTP_NS, 'request-headers') : RequestHeadersElement,
    }

ExtFunctions = {
    (HTTP_NS, 'local-path'): LocalPath,
    (HTTP_NS, 'get-query-args'): GetQueryArguments,
    }

