########################################################################
#
# File Name:            Repository.py
#
"""
XSLT and XPath extensions supporting the 4SS repository API
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import sha
from Ft.Lib import boolean

from Ft.Xml.XPath import Conversions
from Ft.Xml.Xslt import XsltElement, ContentInfo, AttributeInfo, RtfWriter
from Ft.Server import FTSERVER_NAMESPACE, RESERVED_NAMESPACE
from Ns import SCORE_NS

# This element was obviously not being used as GetRepository arguments
# have been changed awhile ago.
##from Ft.Server.Common import ClAuthentcate
##from Ft.Server.Server.SCore import GetRepository
##class LoginPromptElement(XsltElement):
##    #Note, this is only to be used from command-line 4XSLT, and not from within 4SS repo

##    content = ContentInfo.Empty
##    legalAttrs = {}

##    doesPrime = 1
##    doesIdle = 1

##    def prime(self, processor, context):
##        user_name, passwd = ClAuthenticate.UserAuthenticate()
##        processor._repository = GetRepository(user_name, passwd)
##        processor.commitFlag = 1
##        return

##    def idle(self, processor):
##        if processor.commitFlag:
##            processor._repository.txCommit()
##        else:
##            processor._repository.txRollback()
##        return


def RepoUserName(context):
    repo = context.processor._repository
    return repo.getUserName() or ''


class TextSearchRepositoryElement(XsltElement):

    content = ContentInfo.Empty

    legalAttrs = {
        'query' : AttributeInfo.StringAvt(required=1, description='The string to search for'),
        }

    def instantiate(self, context, processor):
        context.setProcessState(self)

        queryString = Conversions.StringValue(self._query.evaluate(context))

        repo = context.processor._repository
        results = repo.searchDocuments(queryString)

        processor.writers[-1].startElement('Results')
        for result in results:
            processor.writers[-1].startElement('Match')
            processor.writers[-1].text(result)
            processor.writers[-1].endElement('Match')

        processor.writers[-1].endElement('Results')
        return (context,)


class TextReindexRepositoryElement(XsltElement):

    content = ContentInfo.Empty
    legalAttrs = {}

    def instantiate(self, context, processor):
        repo = context.processor._repository
        repo.reindex()
        return (context,)


#FIXME: This needs some hard thought.  After all, XSLT can be invoked outside of HTTP
class CreateSessionElement(XsltElement):
    """
    Create a new session
    """
    content = ContentInfo.Empty
    legalAttrs = {
        'key' : AttributeInfo.StringAvt(description='The session key to use',
                                        default=''),
        #'id' : AttributeInfo.StringAvt(description='The session ID to attach'),
        }
    
    def instantiate(self, context, processor):
        from Ft.Server.Server.Http import Session
        from Ft.Server import RESERVED_NAMESPACE
        repo = context.processor._repository
        id = repo.createSession(self._key.evaluate(context))
        handler = processor.extensionParams[(RESERVED_NAMESPACE, 'handler')]
        processor.extensionParams[(RESERVED_NAMESPACE, 'sessionId')] = id
        handler.session = Session.Create(id, handler.session_method)
        processor.writers[-1].text(str(id))
        print "Session set to", id
        return (context,)


def HasSession(context):
    """
    See if this connection is in a session
    """
    repo = context.processor._repository
    if repo.hasSession():
        return boolean.true
    return boolean.false


def SessionInvalidate(context):
    try:
        repo = context.processor._repository
        if not repo.hasSession():
            return boolean.false
        else:
            repo.invalidateSession()
            return boolean.true
    except:
        return boolean.false

    
def GetSessionData(context, key):
    """Session data is stored under a session ID (a UUID string) and takes the
    form of zero or more key-value pairs. Each key is a unique string, and each
    value is a node-set.

    This XSLT extension function coerces the given key to its string-value
    and returns from the current session the node-set that is associated with
    that key. If the key does not exist in the current session, or if there is
    no current session, then an empty node-set is returned. There is no way to
    distinguish between an empty node-set as an indication of such an error and
    an empty node-set as the legitimate value for the given key."""

    key = Conversions.StringValue(key)
    repo = context.processor._repository
    rv = []
    if repo.hasSession():
        try:
            data = repo.getSessionData(key)
            if data is not None:
                print "Session data found for key '%s'" % key
                if type(data) != type([]):
                    print "Error: Session data was not a node-set! It was: %s" % repr(data)
                else:
                    print "Returning %s nodes of Session Data" % len(data)
                    rv = data
            else:
                print "There is a current Session, but it has no data for key '%s'." % key
        except:
            print "There was an error getting the Session data."
    else:
        print "There is no current Session."
    return rv


def SetSessionData(context, key, val):
    """Session data is stored under a session ID (a UUID string) and takes the
    form of zero or more key-value pairs. Each key is a unique string, and each
    value is a node-set.

    This XSLT extension function takes its value argument and stores it in the
    current session, if any, as the value of the given key. If the value is a
    node-set, it is stored as-is. If the value is any other type, including
    result tree fragment, it is converted to a single text node encapsulating
    the string-value of the original object.

    A boolean is returned to indicate success; it will generally only be false
    if there is no current session."""

    key = Conversions.StringValue(key)
    pval = Conversions.StringValue(val)
    if type(val) == type([]):
        pass
    elif hasattr(val, 'nodeType'):
        val = [val]
    else:
        val = [RtfWriter.Text(pval)]

    try:
        repo = context.processor._repository
        if not repo.hasSession():
            print "SetSessionData() called but there is no session!"
            return boolean.false
        else:
            print "Setting Session Data: %s = %s" % (key, repr(val))
            repo.setSessionData(key, val)
            print "Getting it right away: %s" % repr(repo.getSessionData(key))
        return boolean.true
    except:
        raise
        print "SetSessionData(): There was a problem! Session data may not be set."
        return boolean.false


def DeleteSessionData(context, key):
    key = Conversions.StringValue(key)
    try:
        repo = context.processor._repository
        if not repo.hasSession():
            return ''
        print "Remove Session Data for %s" % (key)
        repo.deleteSessionData(key)
        return ''
    except:
        return ''


def HashString(context,rawString):
    rstring = Conversions.StringValue(rawString)
    return sha.new(rstring).hexdigest()


class RollbackElement(XsltElement):

    content = ContentInfo.Empty

    legalAttrs = {
        }

    def instantiate(self, context, processor):
        context.setProcessState(self)
        processor.extensionParams[(RESERVED_NAMESPACE,'rollback')] = 1
        return (context,)


ExtFunctions = {
    (SCORE_NS, 'repo-user-name'): RepoUserName,
    (SCORE_NS, 'has-session'): HasSession,
    #(NS, 'create'): SessionCreate,
    (SCORE_NS, 'session-invalidate'): SessionInvalidate,
    #(NS, 'reattach'): SessionReattach,
    (SCORE_NS, 'get-session-data'): GetSessionData,
    (SCORE_NS, 'set-session-data'): SetSessionData,
    (SCORE_NS, 'delete-session-data'): DeleteSessionData,
    (SCORE_NS, 'hash-string'): HashString,
}

ExtElements = {
    #(SCORE_NS, 'login-prompt'): LoginPromptElement,
    (SCORE_NS, 'rollback'): RollbackElement,
    (SCORE_NS, 'create-session'): CreateSessionElement,
    (SCORE_NS, 'repo-text-search'): TextSearchRepositoryElement,
    (SCORE_NS, 'repo-text-reindex'): TextReindexRepositoryElement,
}

