#Stefan Seefeld <seefeld@sympatico.ca> has trouble with the docbook stylesheets

import os, re
from Xml.Xslt import test_harness

source_1 = """\
<book>
  <chapter>
    <title>Chapter</title>
    <sect1>
      <title>Sect1</title>
      <sect2>
        <title>Sect2</title>
      </sect2>
    </sect1>
  </chapter>
</book>
"""

# NOTE: there is a '%s' for the actual version of docbook used
expected_1 = """\
<html><head><meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1"><title></title><meta name="generator" content="DocBook XSL Stylesheets V%s"></head><body bgcolor="white" text="black" link="#0000FF" vlink="#840084" alink="#0000FF"><div class="book" id="id1"><div class="titlepage"><hr></div><div class="toc"><p><b>Table of Contents</b></p><dl><dt>1. <a href="#id4">Chapter</a></dt><dd><dl><dt><a href="#id3">Sect1</a></dt><dd><dl><dt><a href="#id2">Sect2</a></dt></dl></dd></dl></dd></dl></div><div class="chapter"><div class="titlepage"><div><h2 class="title"><a name="id4"></a>Chapter 1. Chapter</h2></div></div><div class="toc"><p><b>Table of Contents</b></p><dl><dt><a href="#id3">Sect1</a></dt><dd><dl><dt><a href="#id2">Sect2</a></dt></dl></dd></dl></div><div class="sect1"><a name="id3"></a><div class="titlepage"><div><h2 class="title" style="clear: both"><a name="id3"></a>Sect1</h2></div></div><div class="sect2"><a name="id2"></a><div class="titlepage"><div><h3 class="title"><a name="id2"></a>Sect2</h3></div></div></div></div></div></div></body></html>"""

def Test(tester):
    tester.startTest('Checking for DocBook stylesheets')
    dir = os.path.dirname(__file__)
    docbook_dir = os.path.join(dir, 'docbook')
    if not os.access(docbook_dir, os.F_OK):
        tester.warning(
            "You need Norm Walsh's Docbook stylesheet package for this test\n"
            "You can either ignore this, or you can install dbxXXX.zip\n"
            "inside the %r directory\n"
            "where XXX is the version number, which would be at least 144."
            % dir)
        tester.testDone()
        return

    VERSION = open(os.path.join(docbook_dir, 'VERSION')).read()
    match = re.search(r'>\s*(\d+\.\d+)\s*<', VERSION)
    if not match:
        tester.warning("Unable to determine version of DocBook stylesheets")
        tester.testDone()
        return
    version = match.group(1)
    tester.testDone()

    source = test_harness.FileInfo(string=source_1)
    expected = expected_1 % version
    sheet = test_harness.FileInfo(uri='Xml/Xslt/Borrowed/docbook/html/docbook.xsl')
    test_harness.XsltTest(tester, source, [sheet], expected)
    return
