########################################################################
#
# File Name:            FtMiniDom.py
#
#
"""
An abstraction of minidom so it works in 4XSLT

WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

def _new_attribute(klass, name, method):
    setattr(klass, '_get_%s' % name, method)
    try:
        setattr(klass, name, property(method))
    except NameError:
        # property builtin not defined (pre-2.2)
        pass
    return

import sys
import xml
import xml.dom.minidom
from xml.dom.minidom import *
    
from Ft.Xml import EMPTY_PREFIX, EMPTY_NAMESPACE


# We currently support 0.7.1 and 0.8 of PyXML
#And Python 2.2 and 2.1 (which have older version of xml)
#Python 2.2 comes with PyXML 0.7.1
#Python 2.1 comes with PyXML 0.6.6

# Use a float to represent version
version_info = getattr(xml, 'version_info', (0, 0, 0))
pyxml_version = version_info[1] + version_info[2]/10.0
del version_info

# -- PyXML pre-0.8.0 ---------------------------------------------------

if pyxml_version < 8.0:

    def Node__get_previousSibling(self):
        if self.parentNode:
            if self in self.parentNode.childNodes:
                index = self.parentNode.childNodes.index(self) -1
                if index == -1:
                    return None
                return self.parentNode.childNodes[index]
        return None
    _new_attribute(Node, 'previousSibling', Node__get_previousSibling)
    del Node__get_previousSibling

    def Node__get_nextSibling(self):
        if self.parentNode:
            if self in self.parentNode.childNodes:
                index = self.parentNode.childNodes.index(self) +1
                if index == len(self.parentNode.childNodes):
                    return None
                return self.parentNode.childNodes[index]
        return None
    _new_attribute(Node, 'nextSibling', Node__get_nextSibling)
    del Node__get_nextSibling

    def Node__get_prefix(self):
        #If not defined it is null
        return None
    _new_attribute(Node, 'prefix', Node__get_prefix)
    del Node__get_prefix

    def Node__get_localName(self):
        #If not defined it is null
        return None
    _new_attribute(Node, 'localName', Node__get_localName)
    del Node__get_localName

    OldInsertBefore = Node.insertBefore
    def Node_insertBefore(self, newChild, refChild):
        if newChild.nodeType == self.DOCUMENT_FRAGMENT_NODE:
            for c in newChild.childNodes[:]:
                self.insertBefore(c,refChild)
        return OldInsertBefore(self,newChild,refChild)
    Node.insertBefore = Node_insertBefore
    del Node_insertBefore

    OldAppendChild = Node.appendChild
    def Node_appendChild(self, newChild):
        if newChild.nodeType == self.DOCUMENT_FRAGMENT_NODE:
            for c in newChild.childNodes[:]:
                self.appendChild(c)
        return OldAppendChild(self,newChild)
    Node.appendChild = Node_appendChild
    del Node_appendChild

    ### DocumentType Additions ###

    def DocumentType__get_nodeName(self):
        return self.name
    _new_attribute(DocumentType, 'nodeName', DocumentType__get_nodeName)
    del DocumentType__get_nodeName


    ### Attr Override ###

    class FtAttr(Attr):

        def __init__(self, qName, namespaceURI=EMPTY_NAMESPACE, localName=None,
                     prefix=EMPTY_PREFIX):
            self.__class__.__bases__[0].__init__(self, qName, namespaceURI,
                                                 localName, prefix)
            text = Text('')
            self.appendChild(text)

        def __setattr__(self,name,value):
            dict = self.__dict__
            if name == 'value':
                dict['childNodes'][0].data = value
                dict['nodeValue'] = value
            dict[name] = value

        def _get_specified(self):
            #print 'FtAttr.specified'
            return 0

        try:
            specified = property(_get_specified)
        except NameError:
            # property builtin not defined (pre-2.2)
            pass

    Attr = FtAttr
    xml.dom.minidom.Attr = Attr
    del FtAttr


    class FtText(Text):
        childNodes = []
        def __setattr__(self, name, value):
            dict = self.__dict__
            if name == 'data' or name == 'nodeValue':
                dict['data'] = dict['nodeValue'] = value
            else:
                dict[name] = value
            return
    Text = FtText
    xml.dom.minidom.Text = Text
    del FtText


    class FtComment(Comment, CharacterData):
        childNodes = []
        def __setattr__(self,name,value):
            dict = self.__dict__
            if name == 'data' or name == 'nodeValue':
                dict['data'] = dict['nodeValue'] = value
            else:
                dict[name] = value
            return
    Comment = FtComment
    xml.dom.minidom.Comment = Comment
    del FtComment


    class FtProcessingInstruction(ProcessingInstruction):
        def __setattr__(self, name, value):
            dict = self.__dict__
            if name == 'data' or name == 'nodeValue':
                dict['data'] = dict['nodeValue'] = value
            elif name == 'target' or name == 'nodeName':
                dict['target'] = dict['nodeName'] = value
            else:
                dict[name] = value
            return
    ProcessingInstruction = FtProcessingInstruction
    xml.dom.minidom.ProcessingInstruction = ProcessingInstruction
    del FtProcessingInstruction


    class FtDocument(Document):

        def importNode(self,node,deep=0):
            return CloneNode(node,deep,self)

        def createDocumentFragment(self):
            d = DocumentFragment()
            d.ownerDocument = self
            return d

    Document = FtDocument
    xml.dom.minidom.Document = Document
    del FtDocument

    class DOMImplementation:

        _features = {'CORE' : 2.0,
                     'XML' : 2.0,
                     }

        def hasFeature(self, feature, version=''):
            fv = self._features.get(feature.upper())
            if fv:
                if version:
                    try:
                        return float(version) == fv
                    except:
                        return 0
                return 1
            return 0

        def createDocument(self, namespaceURI, qualifiedName, doctype):
            doc = Document()
            doc.doctype = doctype
            if doctype:
                doc.appendChild(doctype)
                doctype.parentNode = doctype.ownerDocument = doc

            if qualifiedName is not None:
                # Non-standard addition to allow for empty documents
                element = doc.createElementNS(namespaceURI, qualifiedName)
                doc.appendChild(element)
            doc.implementation = self
            return doc

        def createDocumentType(self, qualifiedName, publicId, systemId):
            doctype = DocumentType(qualifiedName)
            doctype.publicId = publicId
            doctype.systemId = systemId
            doctype.internalSubset = None
            return doctype

    implementation = DOMImplementation()
    Document.implementation = implementation

    #Set the node Names
    Document.nodeName = '#document'
    DocumentFragment.nodeName = '#document-fragment'
    Text.nodeName ='#text'
    Comment.nodeName = '#comment'

    DocumentType.notations = {}
    DocumentType.entities = {}
    DocumentType.internalSubset = ''




    def _nssplit(qualifiedName):
        fields = qualifiedName.split(':', 1)
        if len(fields) == 2:
            return fields
        elif len(fields) == 1:
            return (EMPTY_PREFIX, fields[0])


    xml.dom.minidom._nssplit = _nssplit


    def CloneNode(node,deep,newOwnerDocument):
        """
        Clone a node and give it the new owner document.
        Called by Node.cloneNode and Document.importNode
        """
        if node.nodeType == Node.ELEMENT_NODE:
            element = newOwnerDocument.createElementNS(node.namespaceURI,
                                                       node.nodeName)
            for attr in node.attributes.values():
                element.setAttributeNS(attr.namespaceURI,attr.nodeName,attr.value)

            if deep:
                for child in node.childNodes:
                    clone = CloneNode(child,deep,newOwnerDocument)
                    element.appendChild(clone)

            return element
        elif node.nodeType == Node.TEXT_NODE:
            text = newOwnerDocument.createTextNode(node.data)
            return text
        elif node.nodeType == PROCESSING_INSTRUCTION_NODE:
            pi = newOwnerDocument.createProcessingInstruction(node.target,node.data)
            return pi
        elif node.nodeType == COMMENT_NODE:
            comment = newOwnerDocument.createComment(node.data)
            return comment
        else:
            raise Exception("Cannot clone node %s" % repr(node))

# -- PyXML 0.8.0+ ------------------------------------------------------

if pyxml_version >= 8.0:
    
    def Text__init__(self, data=''):
        if data:
            self.data = data
    xml.dom.minidom.Text.__init__ = Text__init__
    del Text__init__

    def ReadOnlySequentialNamedNodeMap_keys(self):
        return map(lambda node: (node.namespaceURI, node.localName), self._seq)
    xml.dom.minidom.ReadOnlySequentialNamedNodeMap.keys = ReadOnlySequentialNamedNodeMap_keys
    del ReadOnlySequentialNamedNodeMap_keys

    implementation = xml.dom.minidom.Document.implementation

# -- PyXML 0.8.0 only --------------------------------------------------

if pyxml_version == 8.0:
    from xml.dom.minidom import _nssplit
    
    # Required to add doctype node to childNodes
    def DOMIMP_createDocument(self, namespaceURI, qualifiedName, doctype):
        if doctype and doctype.parentNode is not None:
            raise xml.dom.WrongDocumentErr(
                "doctype object owned by another DOM tree")

        doc = Document()

        add_root_element = not (namespaceURI is None
                                and qualifiedName is None
                                and doctype is None)

        if not qualifiedName and add_root_element:
            # The spec is unclear what to raise here; SyntaxErr
            # would be the other obvious candidate. Since Xerces raises
            # InvalidCharacterErr, and since SyntaxErr is not listed
            # for createDocument, that seems to be the better choice.
            # XXX: need to check for illegal characters here and in
            # createElement.

            # DOM Level III clears this up when talking about the return value
            # of this function.  If namespaceURI, qName and DocType are
            # Null the document is returned without a document element
            # Otherwise if doctype or namespaceURI are not None
            # Then we go back to the above problem
            raise xml.dom.InvalidCharacterErr("Element with no name")

        if add_root_element:
            prefix, localname = _nssplit(qualifiedName)
            if prefix == "xml" \
               and namespaceURI != "http://www.w3.org/XML/1998/namespace":
                raise xml.dom.NamespaceErr("illegal use of 'xml' prefix")
            if prefix and not namespaceURI:
                raise xml.dom.NamespaceErr(
                    "illegal use of prefix without namespaces")
            element = doc.createElementNS(namespaceURI, qualifiedName)
            if doctype:
                doc.appendChild(doctype)
            doc.appendChild(element)

        if doctype:
            doctype.parentNode = doctype.ownerDocument = doc

        doc.doctype = doctype
        doc.implementation = self
        return doc
    xml.dom.minidom.DOMImplementation.createDocument = DOMIMP_createDocument
    del DOMIMP_createDocument

# -- PyXML 0.8.1 only --------------------------------------------------

if pyxml_version == 8.1:

    
    # Required to enable cloning of elements with attributes
    def _clone_node(node, deep, newOwnerDocument):
        """
        Clone a node and give it the new owner document.
        Called by Node.cloneNode and Document.importNode
        """
        if node.ownerDocument.isSameNode(newOwnerDocument):
            operation = xml.dom.UserDataHandler.NODE_CLONED
        else:
            operation = xml.dom.UserDataHandler.NODE_IMPORTED
        if node.nodeType == Node.ELEMENT_NODE:
            clone = newOwnerDocument.createElementNS(node.namespaceURI,
                                                     node.nodeName)
            for attr in node.attributes.values():
                clone.setAttributeNS(attr.namespaceURI, attr.nodeName,
                                     attr.value)
                a = clone.getAttributeNodeNS(attr.namespaceURI, attr.localName)
                a.specified = attr.specified

            if deep:
                for child in node.childNodes:
                    c = _clone_node(child, deep, newOwnerDocument)
                    clone.appendChild(c)

        elif node.nodeType == Node.DOCUMENT_FRAGMENT_NODE:
            clone = newOwnerDocument.createDocumentFragment()
            if deep:
                for child in node.childNodes:
                    c = _clone_node(child, deep, newOwnerDocument)
                    clone.appendChild(c)

        elif node.nodeType == Node.TEXT_NODE:
            clone = newOwnerDocument.createTextNode(node.data)
        elif node.nodeType == Node.CDATA_SECTION_NODE:
            clone = newOwnerDocument.createCDATASection(node.data)
        elif node.nodeType == Node.PROCESSING_INSTRUCTION_NODE:
            clone = newOwnerDocument.createProcessingInstruction(node.target,
                                                                 node.data)
        elif node.nodeType == Node.COMMENT_NODE:
            clone = newOwnerDocument.createComment(node.data)
        elif node.nodeType == Node.ATTRIBUTE_NODE:
            clone = newOwnerDocument.createAttributeNS(node.namespaceURI,
                                                       node.nodeName)
            clone.specified = True
            clone.value = node.value
        elif node.nodeType == Node.DOCUMENT_TYPE_NODE:
            assert node.ownerDocument is not newOwnerDocument
            operation = xml.dom.UserDataHandler.NODE_IMPORTED
            clone = newOwnerDocument.implementation.createDocumentType(
                node.name, node.publicId, node.systemId)
            clone.ownerDocument = newOwnerDocument
            if deep:
                clone.entities._seq = []
                clone.notations._seq = []
                for n in node.notations._seq:
                    notation = Notation(n.nodeName, n.publicId, n.systemId)
                    notation.ownerDocument = newOwnerDocument
                    clone.notations._seq.append(notation)
                    if hasattr(n, '_call_user_data_handler'):
                        n._call_user_data_handler(operation, n, notation)
                for e in node.entities._seq:
                    entity = Entity(e.nodeName, e.publicId, e.systemId,
                                    e.notationName)
                    entity.actualEncoding = e.actualEncoding
                    entity.encoding = e.encoding
                    entity.version = e.version
                    entity.ownerDocument = newOwnerDocument
                    clone.entities._seq.append(entity)
                    if hasattr(e, '_call_user_data_handler'):
                        e._call_user_data_handler(operation, n, entity)
        else:
            # Note the cloning of Document and DocumentType nodes is
            # implemenetation specific.  minidom handles those cases
            # directly in the cloneNode() methods.
            raise xml.dom.NotSupportedErr("Cannot clone node %s" % repr(node))

        # Check for _call_user_data_handler() since this could conceivably
        # used with other DOM implementations (one of the FourThought
        # DOMs, perhaps?).
        if hasattr(node, '_call_user_data_handler'):
            node._call_user_data_handler(operation, node, clone)
        return clone
    xml.dom.minidom._clone_node = _clone_node
    del _clone_node
    
# -- PyXML regardless of version ---------------------------------------

# Required to enable NS-aware dictionary interface
def NNM__len__(self):
    return len(self._attrsNS)
NamedNodeMap.__len__ = NNM__len__
del NNM__len__

def NNM_has_key(self,key):
    return self._attrsNS.has_key(key)
NamedNodeMap.has_key = NNM_has_key
del NNM_has_key

def NNM_values(self):
    return self._attrsNS.values()
NamedNodeMap.values = NNM_values
del NNM_values

def NNM_items(self):
    return self._attrsNS.items()
NamedNodeMap.items = NNM_items
del NNM_items

NamedNodeMap.keys = NamedNodeMap.keysNS

def NNM_get(self,key,default=None):
    return self._attrsNS.get(key,default)
NamedNodeMap.get = NNM_get
del NNM_get

def NNM_repr(self):
    return repr(self._attrsNS)
NamedNodeMap.__repr__ = NNM_repr
del NNM_repr

# Remove getElementById from Document interface until we implement the
# same functionality that expatbuilder uses for storing DTD information
# on elements
if hasattr(xml.dom.minidom.Document, 'getElementById'):
    del xml.dom.minidom.Document.getElementById
