########################################################################
#
# File Name:            ValReader.py
#
#
"""
The validating reader
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright 2001-2002 Fourthought Inc, USA.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml import ReaderException

import Handler

try:
    from xml.parsers import xmlproc

    class ValReader(Handler.Handler):

        ### Reader Interface ###

        def fromSrc(self, source):
            self.prepareParser()
            self.initHandler(source)

            # Parse it
            self.parse(source)
            
            self.reset()
            return self.getRootNode()

        def prepareParser(self):
            # We do this here due to the long import time
            from xml.parsers.xmlproc import xmlval, xmlproc

            # Replace xmlproc's default uri resolution.  We handle it
            # in the resolve_*_pubid() methods.
            from xml.parsers.xmlproc import xmlutils, dtdparser
            xmlutils.join_sysids = lambda base, url: url
            xmlproc.join_sysids = lambda base, url: url
            dtdparser.join_sysids = lambda base, url: url

            self.parser = xmlval.XMLValidator()

            self.parser.set_application(self)
            self.parser.set_error_handler(self)
            self.parser.set_pubid_resolver(self)
            self.parser.set_inputsource_factory(self)
            
            # For validation
            self.parser.set_read_external_subset(1)
            return

        def parse(self, source):
            self.parser.parse_resource(source)

        def reset(self):
            self.parser = None
            return

        ### XMLPROC INTERFACES ###

        # --- InputSourceFactory methods

        def create_input_source(self, systemId):
            # systemId as returned from resolve_*_pubid()
            return systemId.stream

        # --- PubIdResolver methods

        def resolve_pe_pubid(self, publicId, systemId):
            #FIXME: this is a quick hack to get around a bug in this class.
            #the parser's InputSource is not kept up to date as entities
            #are resolved.  For some reason the locator's is
            if isinstance(self._locator.get_current_sysid(), type("")):
                resolver = self.parser.get_current_sysid()
            else:
                resolver = self._locator.get_current_sysid()
            #resolver = self._locator.get_current_sysid()
            return resolver.resolve(systemId, publicId, 'PARAMETER_ENTITY')

        def resolve_doctype_pubid(self, publicId, systemId):
            #FIXME: this is a quick hack to get around a bug in this class.
            #the parser's InputSource is not kept up to date as entities
            #are resolved.  For some reason the locator's is
            if isinstance(self._locator.get_current_sysid(), type("")):
                resolver = self.parser.get_current_sysid()
            else:
                resolver = self._locator.get_current_sysid()
            #resolver = self._locator.get_current_sysid()
            return resolver.resolve(systemId, publicId, 'DOCTYPE_DECL')

        def resolve_entity_pubid(self, publicId, systemId):
            #FIXME: this is a quick hack to get around a bug in this class.
            #the parser's InputSource is not kept up to date as entities
            #are resolved.  For some reason the locator's is
            if isinstance(self._locator.get_current_sysid(), type("")):
                resolver = self.parser.get_current_sysid()
            else:
                resolver = self._locator.get_current_sysid()
            #resolver = self._locator.get_current_sysid()
            return resolver.resolve(systemId, publicId, 'EXTERNAL_ENTITY')
            
        # --- Application methods

        def set_locator(self, locator):
            self._locator = locator
    
        def doc_start(self):
            pass

        def doc_end(self):
            pass

        def handle_comment(self, data):
            self.comment(data)

        def handle_start_tag(self, name, attrs):
            self.startElement(name, attrs)

        def handle_end_tag(self, name):
            self.endElement(name)

        def handle_data(self, data, start, end):
            self.characters(data[start:end])

        def handle_ignorable_data(self, data, start, end):
            pass

        def handle_pi(self, target, data):
            self.processingInstruction(target, data)

        def handle_doctype(self, root, pubId, sysId):
            pass

        def set_entity_info(self, xmlver, enc, sddecl):
            pass

        # --- ErrorHandler methods

        def get_locator(self):
            return self._locator

        def fatal(self, message):
            raise ReaderException(ReaderException.XML_PARSE_ERROR,
                                  self._locator.get_current_sysid().uri,
                                  self._locator.get_line(),
                                  self._locator.get_column(),
                                  message)
        error = fatal
        warning = fatal



except ImportError:
    # Create a dummy reader class
    class ValReader:
        def __init__(self, *varargs, **varkw):
            from Ft.Xml import CheckVersion
            CheckVersion('a validating parser')


def valParse(src):
    r = ValReader()
    return r.fromSrc(src)
