from HtmlPrinter import HtmlPrinter

class HtmlPrettyPrinter(HtmlPrinter):

    # The amount of indent for each level of nesting
    indent = '  '

    def __init__(self, stream, encoding):
        HtmlPrinter.__init__(self, stream, encoding)
        self.level = 0

        # indenting control variables
        self.isInline = [1]  # prevent newline before first element
        self.inNoIndent = [0]
        return

    def startElement(self, tagName, namespaces, attributes):
        if not self.isInline[-1] and not self.inNoIndent[-1]:
            self.writeAscii('\n' + (self.indent * self.level))

        HtmlPrinter.startElement(self, tagName, namespaces, attributes)

        # Setup indenting rules for this element
        name = tagName.lower()
        self.isInline.append(self.inlineElements.has_key(name))
        self.inNoIndent.append(self.noIndentElements.has_key(name))
        self.level += 1
        return

    def endElement(self, tagName):
        # Undo changes to indenting rules for this element
        self.level -= 1
        inline = self.isInline.pop() 
        no_indent = self.inNoIndent.pop()

        if not inline and not no_indent:
            self.writeAscii('\n' + (self.indent * self.level))

        HtmlPrinter.endElement(self, tagName)
        return

    def text(self, data):
        if self.inNoIndent[-1]:
            HtmlPrinter.text(self, data)
        else:
            lines = text.split('\n')
            HtmlPrinter.text(self, lines[0])
            for line in lines[1:]:
                self.writeAscii('\n' + (self.indent * self.level))
                HtmlPrinter.text(self, line)
        return

    def processingInstruction(self, target, data):
        # try to indent
        if not self.isInline[-1] and not self.inNoIndent[-1]:
            self.writeAscii('\n' + (self.indent * self.level))
        HtmlPrinter.processingInstruction(self, target, data)
        return

    def comment(self, data):
        # try to indent
        if not self.isInline[-1] and not self.inNoIndent[-1]:
            self.writeAscii('\n' + (self.indent * self.level))
        HtmlPrinter.comment(self, data)
        return

    # inline elements (<td> added to list)
    inlineElements = {}
    for name in ['tt', 'i', 'b', 'u', 's', 'strike', 'big', 'small', 'em',
                 'strong', 'dfn', 'code', 'samp', 'kbd', 'var', 'cite',
                 'abbr', 'acronym', 'a', 'img', 'applet', 'object', 'font',
                 'basefont', 'script', 'map', 'q', 'sub', 'sup', 'span',
                 'bdo', 'iframe', 'input', 'select', 'textarea', 'label',
                 'button', 'td']:
        inlineElements[name] = 1

    # preformatted elements, no additional whitespace is added
    noIndentElements = {}
    for name in ['script', 'style', 'pre', 'textarea']:
        noIndentElements[name] = 1

    del name
    
