########################################################################
#
# File Name:   Context.py
#
# Docs:        http://docs.4suite.org/XPATH/Context.py.html
#
"""
The context of an XPath expression.
WWW: http://4suite.org/XPATH        e-mail: support@4suite.org

Copyright (c) 2000-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import types
from Ft.Xml import Domlette
import CoreFunctions, BuiltInExtFunctions

class Context:
    functions = CoreFunctions.CoreFunctions.copy()
    functions.update(BuiltInExtFunctions.ExtFunctions)
    
    def __init__(self,
                 node,
                 position=1,
                 size=1,
                 varBindings=None,
                 processorNss=None,
                 extModuleList=None,
                 extFunctionMap=None):
        self.node = node
        self.position = position
        self.size = size
        self.varBindings = varBindings or {}
        self.processorNss = processorNss or {}

        self._documentIndex = {}

        # This may get mutated during processing
        functions = self.functions.copy()

        # Search the extension modules for defined functions
        if extModuleList:
            for module in extModuleList:
                if module:
                    if not isinstance(module, types.ModuleType):
                        module = __import__(module, {}, {}, ['ExtFunctions'])
                    
                    if hasattr(module, 'ExtFunctions'):
                        functions.update(module.ExtFunctions)

        # Add functions given directly
        if extFunctionMap:
            functions.update(extFunctionMap)
        self.functions = functions
        return

    def __repr__(self):
        return "<Context at 0x%x: Node=%s, Postion=%d, Size=%d>" % (
            id(self), self.node, self.position, self.size)

    def addFunction(self, expandedName, function):
        if not callable(function):
            raise TypeError("function must be a callable object")
        self.functions[expandedName] = function
        return

    # Used for SortDocOrder with a nodeset containing multiple documents
    def addDocument(self, document):
        if not self._documentIndex.has_key(document):
            self._documentIndex[document] = len(self._documentIndex)
        return

    # Used for SortDocOrder with a nodeset containing multiple documents
    def compareDocuments(self, a, b):
        # Make sure the documents we are comparing have an index
        a = self._documentIndex.setdefault(a, len(self._documentIndex))
        b = self._documentIndex.setdefault(b, len(self._documentIndex))
        return cmp(a, b)
    
    def copy(self):
        return (self.node, self.position, self.size)

    def set(self, state):
        self.node, self.position, self.size = state
        return

    def clone(self):
        newobj = self.__class__(self, self.node, self.position, self.size)
        newobj.varBindings = self.varBindings.copy()
        newobj.processorNss = self.processorNss.copy()
        return newobj

