#from Ft.Xml.Xslt import Error

try:
    import os, gettext
    locale_dir = os.path.split(__file__)[0]
    gettext.install('4Suite', locale_dir)
#except ImportError, IOError:
#Note, 1.5.2 has gettext, but no install
except (ImportError,AttributeError,IOError):
    def _(msg):
        return msg

POSITION_INFO = _('In stylesheet %s, line %s, column %s:\n%s')
EXPRESSION_POSITION_INFO = _('In stylesheet %s, line %s, column %s in "%s":\n%s')
XSLT_EXPRESSION_POSITION_INFO = _('%s\nThe error occurred in the expression "%s".')

class Error:
    INTERNAL_ERROR = 1
    PATTERN_SYNTAX = 2
    PATTERN_SEMANTIC = 3
    NO_STYLESHEET = 4
    STYLESHEET_MISSING_VERSION = 5
    LITERAL_RESULT_MISSING_VERSION = 6
    STYLESHEET_PARSE_ERROR = 7
    SOURCE_PARSE_ERROR = 8
    XSL_STYLESHEET_NOT_DOCELEM = 9

    TOP_LEVEL_ELEM_WITH_NULL_NS = 10
    XSLT_ILLEGAL_ELEMENT = 12
    STYLESHEET_ILLEGAL_ROOT = 13
    CIRCULAR_VAR = 14

    # element validation
    ILLEGAL_ELEMENT_CHILD = 20
    ILLEGAL_TEXT_CHILD = 21
    UNDEFINED_PREFIX = 25

    # attribute validation
    MISSING_REQUIRED_ATTRIBUTE = 30
    ILLEGAL_NULL_NAMESPACE_ATTR = 31
    ILLEGAL_XSL_NAMESPACE_ATTR = 32
    INVALID_ATTR_CHOICE = 33
    INVALID_CHAR_ATTR = 34
    INVALID_NUMBER_ATTR = 35
    INVALID_URI_REFERENCE_ATTR = 36
    INVALID_ID_ATTR = 37
    INVALID_QNAME_ATTR = 38
    INVALID_NCNAME_ATTR = 39
    INVALID_PREFIX_ATTR = 40
    INVALID_NMTOKEN_ATTR = 41
    QNAME_BUT_NOT_NCNAME = 42


    # attribute value template
    AVT_SYNTAX = 100
    AVT_EMPTY = 101

    WHEN_AFTER_OTHERWISE = 111
    MULTIPLE_OTHERWISE = 112

    APPLYIMPORTS_WITH_NULL_CURRENT_TEMPLATE = 120

    DUPLICATE_NAMESPACE_ALIAS = 300
    DUPLICATE_TOP_LEVEL_VAR = 301
    KEY_WITH_RTF_CONTEXT = 310

    #xsl:import and xsl:include
    ILLEGAL_IMPORT = 130
    IMPORT_NOT_FOUND = 131
    INCLUDE_NOT_FOUND = 132

    #xsl:choose
    ILLEGAL_CHOOSE_CHILD = 140
    CHOOSE_REQUIRES_WHEN = 141
    CHOOSE_WHEN_AFTER_OTHERWISE = 142
    CHOOSE_MULTIPLE_OTHERWISE = 143

    # xsl:call-template
    ILLEGAL_CALLTEMPLATE_CHILD = 150
    NAMED_TEMPLATE_NOT_FOUND = 151
    DUPLICATE_NAMED_TEMPLATE = 152

    #xsl:template
    ILLEGAL_TEMPLATE_PRIORITY = 160

    #xsl:attribute
    ATTRIBUTE_ADDED_AFTER_ELEMENT = 170
    ATTRIBUTE_MISSING_NAME = 171

    #xsl:element
    UNDEFINED_ATTRIBUTE_SET = 180

    #xsl:for-each
    INVALID_FOREACH_SELECT = 190

    #xsl:value-of
    VALUEOF_MISSING_SELECT = 200

    #xsl:copy-of
    COPYOF_MISSING_SELECT = 210

    #xsl:text
    ILLEGAL_TEXT_CHILD = 220

    #xsl:apply-template
    ILLEGAL_APPLYTEMPLATE_CHILD = 230
    ILLEGAL_APPLYTEMPLATE_MODE = 231
    ILLEGAL_APPLYTEMPLATE_NODESET = 232

    #xsl:when
    WHEN_MISSING_TEST = 240

    # xsl:attribute-set
    ILLEGAL_ATTRIBUTESET_CHILD = 250
    ATTRIBUTESET_REQUIRES_NAME = 251
    CIRCULAR_ATTRIBUTE_SET = 252

    #xsl:param and xsl:variable
    ILLEGAL_PARAM = 260
    ILLEGAL_PARAM_PARENT = 261
    ILLEGAL_SHADOWING = 262
    VAR_WITH_CONTENT_AND_SELECT = 263

    #xsl:message (see also #10000)
    ILLEGAL_MESSAGE_PARENT = 270

    #xsl:processing-instruction
    ILLEGAL_XML_PI = 280

    # xsl:output
    UNKNOWN_OUTPUT_METHOD = 290
    
    # xsl:decimal-format
    DUPLICATE_DECIMAL_FORMAT = 300

    # patterns and expressions
    INVALID_PATTERN = 1000
    INVALID_EXPRESSION = 1001
    INVALID_AVT = 1002

    ILLEGAL_SORT_DATA_TYPE_VALUE = 2010
    ILLEGAL_SORT_CASE_ORDER_VALUE = 2011
    ILLEGAL_SORT_ORDER_VALUE = 2012

    ILLEGAL_NUMBER_GROUPING_SIZE_VALUE = 2020
    ILLEGAL_NUMBER_LEVEL_VALUE = 2021
    ILLEGAL_NUMBER_LETTER_VALUE_VALUE = 2022
    ILLEGAL_NUMBER_FORMAT_VALUE = 2023

    INVALID_NAMESPACE_ALIAS = 2030

    # EXSLT messages use 3000-3999; see Exslt/MessageSource.py

    WRONG_NUMBER_OF_ARGUMENTS = 5000
    WRONG_ARGUMENT_TYPE = 5001
    INVALID_QNAME_ARGUMENT = 5002

    RESTRICTED_OUTPUT_VIOLATION = 7000

    FEATURE_NOT_SUPPORTED = 9999

    STYLESHEET_REQUESTED_TERMINATION = 10000


g_errorMessages = {
    Error.INTERNAL_ERROR: _('There is an internal bug in 4XSLT.  Please report this error code to support@4suite.org: %s'),
    Error.PATTERN_SYNTAX: _('Syntax error in pattern at location %s (XPattern production number: %d).'),
    Error.PATTERN_SEMANTIC: _('Parse tree error in pattern at location %s (XPattern production number: %d, error type: %s, error value: %s, traceback:\n%s'),

    Error.APPLYIMPORTS_WITH_NULL_CURRENT_TEMPLATE: _('apply-imports used where there is no current template.  (see XSLT Spec)'),
    Error.DUPLICATE_NAMESPACE_ALIAS: _('The namespace for "%s" has duplicate namespace aliases defined with the same import precedence.  (see XSLT Spec: 2.6.2)'),
    Error.DUPLICATE_TOP_LEVEL_VAR: _('Top level variable %s has duplicate definitions with the same import precedence.  (see XSLT Spec: 11)'),
    Error.KEY_WITH_RTF_CONTEXT: _('key() must not be invoked when the context node comes from the result tree (probably due to an earlier invokation of node-set()).'),
    Error.XSL_STYLESHEET_NOT_DOCELEM: _('An xsl:stylesheet or xsl:transform element must be the document element.'),
    Error.ILLEGAL_IMPORT: _('xsl:import is not allowed here (xsl:import must be at top level and precede all other XSLT top-level instructions).  (see XSLT Spec: 2.6.2)'),
    Error.INCLUDE_NOT_FOUND: _('Unable to open imported or included stylesheet "%s", using base URI "%s", or all base URIs in the include PATH'),
    Error.STYLESHEET_PARSE_ERROR: _('Stylesheet (%s): XML parse error at line %d, column %d: %s'),
    Error.CIRCULAR_VAR: _('Circular variable reference error (see XSLT Spec: 11.4) for variable or parameter: (%s, %s)'),
    Error.SOURCE_PARSE_ERROR: _('Source document (%s): %s'),

    # xsl:call-template and named templates
    Error.ILLEGAL_CALLTEMPLATE_CHILD: _('call-template child must be with-param., (see XSLT Spec: 6)'),
    Error.NAMED_TEMPLATE_NOT_FOUND: _('Named template "%s" invoked but not defined.'),
    Error.DUPLICATE_NAMED_TEMPLATE : _("Named template '%s' already defined "
                                       "with same import precedence"),


    Error.ILLEGAL_APPLYTEMPLATE_CHILD: _('apply-templates child must be with-'
                                         'param or sort. (see XSLT Spec 5.4)'),
    Error.ILLEGAL_APPLYTEMPLATE_MODE: _('apply-templates mode must be a QName. '
                                        '(see XSLT Spec 5.4)'),
    Error.ILLEGAL_APPLYTEMPLATE_NODESET: _('apply-templates must apply to a '
                                           'node-set.'),

    Error.WHEN_AFTER_OTHERWISE: _('when cannot succeed otherwise.'),
    Error.MULTIPLE_OTHERWISE: _('there cannot be more than one otherwise within a choose.'),
    Error.ILLEGAL_CHOOSE_CHILD: _('choose child must be one or more "when" elements and an optional "otherwise" element., (see XSLT Spec: 9.2)'),
    Error.CHOOSE_WHEN_AFTER_OTHERWISE: _('choose cannot have "when" child after "otherwise" child., (see XSLT Spec: 9.2)'),
    Error.CHOOSE_MULTIPLE_OTHERWISE: _('choose only allowed one "otherwise" child., (see XSLT Spec: 9.2)'),
    Error.CHOOSE_REQUIRES_WHEN: _('choose must have atleast one "when" child., (see XSLT Spec: 9.2)'),
    Error.ILLEGAL_TEXT_CHILD: _('xsl:text cannot have any child elements"., (see XSLT Spec: 7.2)'),

    # xsl:attribute-set (250-259)
    Error.ILLEGAL_ATTRIBUTESET_CHILD: _('attribute-set child must be "attribute"., (see XSLT Spec: 7.1.4)'),
    Error.ATTRIBUTESET_REQUIRES_NAME: _('missing attribute-set required attribute name., (see XSLT Spec: 7.1.4)'),
    Error.CIRCULAR_ATTRIBUTE_SET : _("Circular attribute-set error for '%s'. "
                                     "(see XSLT Spec: 7.1.4)"),


    Error.INVALID_FOREACH_SELECT: _('for-each select attribute must evaluate to a node set (see XSLT Spec: 8)'),

    Error.VALUEOF_MISSING_SELECT: _('missing value-of requried attribute select (see XSLT Spec: 7.6.1)'),
    Error.COPYOF_MISSING_SELECT: _('missing copy-of requried attribute select (see XSLT Spec: 11.3)'),
    Error.WHEN_MISSING_TEST: _('missing when requried attribute test (see XSLT Spec: 9.2)'),

    #Error.TOP_LEVEL_ELEM_WITH_NULL_NS: _(''),
    Error.ILLEGAL_SHADOWING: _('illegal shadowing of %s.  An xsl:param or xsl:variable may not shadow another variable not at top level (see XSLT Spec: 11).'),
    Error.VAR_WITH_CONTENT_AND_SELECT: _('illegal binding of of %s.  An xsl:param or xsl:variable may not have both a select attribute and non-empty content. (see XSLT Spec: 11.2).'),
    Error.ILLEGAL_PARAM: _('xsl:param elements must be the first children of xsl:template (see XSLT Spec: 11).'),
    Error.ILLEGAL_PARAM_PARENT: _('Uri: %s line %s col: %s\n   xsl:param can only appear at top level or as the child of an xsl:template (see XSLT Spec: 11).'),
    Error.STYLESHEET_ILLEGAL_ROOT: _('Illegal Document Root Element "%s" (see XSLT Spec: 2.2).'),

    Error.ILLEGAL_SORT_DATA_TYPE_VALUE: _('The "data-type" attribute of sort must be either "text" or "number" (see XSLT Spec: 10).'),
    Error.ILLEGAL_SORT_CASE_ORDER_VALUE: _('The "case-order" attribute of sort must be either "upper-first" or "lower-first" (see XSLT Spec: 10)'),
    Error.ILLEGAL_SORT_ORDER_VALUE: _('The "order" attribute of sort must be either "ascending" or "descending". (see XSLT Spec: 10)'),
    Error.NO_STYLESHEET: _('No stylesheets to process.'),
    Error.STYLESHEET_MISSING_VERSION: _('Stylesheet "%s", document root element must have a version attribute.  (see XSLT Spec: 2.2)'),
    Error.LITERAL_RESULT_MISSING_VERSION: _('Document root element must have a xsl:version attribute.  (see XSLT Spec: 2.3).'),
    Error.ILLEGAL_TEMPLATE_PRIORITY: _('Invalid priority value for template. (see XSLT Spec: 5.5)'),

    Error.ILLEGAL_NUMBER_GROUPING_SIZE_VALUE: _('The "grouping-size" attribute of number must be an integer. (see XSLT Spec: 7.7.1)'),
    Error.ILLEGAL_NUMBER_LEVEL_VALUE: _('The "level" attribute of number must be "single", "multiple" or "any". (see XSLT Spec: 7.7)'),
    Error.ILLEGAL_NUMBER_FORMAT_VALUE: _('Invalid value for "format" attribute of number. (see XSLT Spec: 7.7)'),
    Error.ILLEGAL_NUMBER_LETTER_VALUE_VALUE: _('The "letter-value" attribute of number must be "alphabetic" or "traditional". (see XSLT Spec: 7.7.1)'),

    Error.INVALID_NAMESPACE_ALIAS: _('Invalid arguments to the namespace-alias instruction. (see XSLT Spec: 7.1.1)'),

    Error.WRONG_NUMBER_OF_ARGUMENTS: _('A built-in or extension function was called with the wrong number of arguments.'),
    Error.WRONG_ARGUMENT_TYPE: _('A built-in or extension function was called with the wrong type of argument(s).'),
    Error.INVALID_QNAME_ARGUMENT: _('A built-in or extension function requiring a QName argument was called with this non-QName value: "%s".'),

    Error.FEATURE_NOT_SUPPORTED: _('4XSLT does not yet support this feature.'),

    Error.ILLEGAL_MESSAGE_PARENT: _('xsl:message cannot be a top-level element. (see XSLT Spec: 2.2)'),

    Error.ILLEGAL_XML_PI: _('A processing instruction cannot be used to output an XML or text declaration. (see XSLT Spec: 7.3)'),

    # xsl:output
    Error.UNKNOWN_OUTPUT_METHOD : _('Unknown output method (%s)'),

    # xsl:decimal-format
    Error.DUPLICATE_DECIMAL_FORMAT : _('Duplicate declaration of decimal-format'
                                       " %s. (XSLT Spec: 12.3)"),

    # attribute value templates
    Error.AVT_SYNTAX: _('Unbalanced curly braces ({}) in attribute value '
                        'template. (see XSLT Spec: 7.6.2)'),
    Error.AVT_EMPTY : _('No expression in attribute value template.'),

    # patterns and expressions
    Error.INVALID_PATTERN: _('Malformed pattern: "%s" in the element at %s, line %s, column %s\n  %s'),
    Error.INVALID_EXPRESSION: _('Malformed expression: "%s" in the element at %s, line %s, column %s\n  %s'),
    Error.INVALID_AVT: _('Malformed attribute value template: "%s" in the element at %s, line %s, column %s\n  %s'),

    Error.ATTRIBUTE_ADDED_AFTER_ELEMENT: _('xsl:attribute instantiated within an element instantiation after a child element has been added. (see XSLT Spec: 7.1.3)'),
    Error.ATTRIBUTE_MISSING_NAME: _('xsl:attribute missing required name attribute. (see XSLT Spec: 7.1.3)'),


    Error.UNDEFINED_ATTRIBUTE_SET: _('Undefined attribute set (%s)'),
    Error.RESTRICTED_OUTPUT_VIOLATION: _('The requested output of element "%s" is forbidden accirding to output restrictions'),


    Error.STYLESHEET_REQUESTED_TERMINATION: _('A message instruction in the Stylesheet requested termination of processing:\n%s'),

    Error.XSLT_ILLEGAL_ELEMENT: _("Illegal element '%s' in XSLT Namespace "
                                  "(see XSLT Spec: 2.1)."),
    Error.ILLEGAL_ELEMENT_CHILD : _("Illegal child '%s' within element '%s'"),
    Error.ILLEGAL_TEXT_CHILD : _("Illegal literal text %s within element '%s'"),

    Error.UNDEFINED_PREFIX : _("Undefined namespace prefix '%s'"),

    # -- attribute validation ------------------------------

    Error.MISSING_REQUIRED_ATTRIBUTE : _("Element '%s' missing required"
                                            " attribute '%s'"),
    Error.ILLEGAL_NULL_NAMESPACE_ATTR : _("Illegal null-namespace attribute "
                                          "'%s' on element '%s'."),
    Error.ILLEGAL_XSL_NAMESPACE_ATTR : _("Illegal xsl-namespace attribute "
                                         "'%s' on element '%s'."),
    Error.INVALID_ATTR_CHOICE : _("Illegal attribute value '%s', must be "
                                  "one of '%s'"),
    Error.INVALID_CHAR_ATTR : _("Invalid char attribute value '%s'"),
    Error.INVALID_NUMBER_ATTR : _("Invalid number attribute value '%s'"),
    Error.INVALID_URI_REFERENCE_ATTR : _("Invalid uri-reference attribute"
                                         "value '%s'"),
    Error.INVALID_ID_ATTR : _("Invalid ID attribute value '%s'"),
    Error.INVALID_QNAME_ATTR : _("Invalid QName attribute value '%s'"),
    Error.INVALID_NCNAME_ATTR : _("Invalid NCName attribute value '%s'"),
    Error.INVALID_PREFIX_ATTR : _("Invalid prefix attribute value '%s'"),
    Error.INVALID_NMTOKEN_ATTR : _("Invalid NMTOKEN attribute value '%s'"),
    Error.QNAME_BUT_NOT_NCNAME : _("QName allowed but not NCName, '%s' found"),
}
