########################################################################
#
# File Name:            ProcessingInstructionElement.py
#
# Documentation:        http://docs.4suite.org/4XSLT/ProcessingInstructionElement.py.html
#
"""
Implementation of the XSLT Spec processing-instruction stylesheet element.
WWW: http://4suite.org/4XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

from Ft.Xml import EMPTY_NAMESPACE
from Ft.Xml.Xslt import XsltElement, XsltRuntimeException, Error, XSL_NAMESPACE
from Ft.Xml.Xslt import CategoryTypes, ContentInfo, AttributeInfo

class ProcessingInstructionElement(XsltElement):
    category = CategoryTypes.INSTRUCTION
    content = ContentInfo.Template
    legalAttrs = {
        'name' : AttributeInfo.NCNameAvt(required=1),
        }

    def instantiate(self, context, processor):
        context.processorNss = self.namespaces
        context.currentInstruction = self

        target = self._name.evaluate(context)
        if target.lower() == 'xml':
            raise XsltRuntimeException(Error.ILLEGAL_XML_PI, self)

        processor.pushResult()
        try:
            for child in self.children:
                context = child.instantiate(context, processor)[0]
        finally:
            result = processor.popResult()

        # FIXME:
        # Per the XSLT spec, it is an error if the result contains
        # non-text nodes. We must either signal it or ignore the
        # non-text nodes altogether, even if they have text in them.

        # Per the spec, PI data can't contain '?>', but we are allowed
        # to replace it with '? >' instead of signalling an error.
        data = result.stringValue.replace(u'?>',u'? >')

        processor.writers[-1].processingInstruction(target, data)
        return (context,)
