########################################################################
#
# File Name:   XsltFunctions.py
#
# Docs:        http://docs.4suite.org/XSLT/XsltFunctions.py.html
#

"""
WWW: http://4suite.org/XSLT        e-mail: support@4suite.org

Copyright (c) 1999-2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import os, cStringIO
from Ft.Lib import boolean
from Ft.Xml import __version__, EMPTY_NAMESPACE
from Ft.Xml.Lib import routines
from Ft.Xml.XPath import Conversions, Util, Types
from Ft.Xml.Xslt import XsltRuntimeException, Error, XSL_NAMESPACE

# is there no better list of implemented core XSLT elements?
from Ft.Xml.Xslt.StylesheetHandler import _ELEMENT_MAPPING

def Document(context, object, nodeSet=None):
    result = []
    sheet = context.processor.stylesheet
    baseUri = context.currentInstruction.baseUri
    if nodeSet:
        baseUri = getattr(nodeSet[0].ownerDocument or nodeSet[0],
                          'refUri', baseUri)

    if nodeSet is None:
        if isinstance(object, Types.NodesetType):
            for curr_node in object:
                result = result + Document(context,
                                           Conversions.StringValue(curr_node),
                                           [curr_node])
        else:
            try:
                # when the call is document(''), we want just the XML doc
                # containing the document('') call, not the entire stylesheet.
                if object == '':
                    uri = baseUri
                else:
                    #FIXME: Discard fragments before checking for dupes
                    uri = sheet._input_source.getUriResolver().normalize(Conversions.StringValue(object), baseUri)


                if context.documents.has_key(uri):
                    # Used the cached DOM
                    result = [context.documents[uri]]
                else:
                    if object == '' and sheet.root.sources.has_key(baseUri):
                        # Use the cached string versions
                        # Create an input source with the same properties
                        # as the document input source.
                        stream = cStringIO.StringIO(sheet.root.sources[baseUri])
                        isrc = context.processor._documentInputSource._clone(stream, baseUri)
                    else:
                        #Create a new input source
                        isrc = context.processor._documentInputSource.resolve(uri)
                    doc = context.processor._docReader.parse(isrc)
                    context.addDocument(doc, uri)
                    result = [doc]
            except (OSError, IOError):
                context.processor.warning("Unable to retrieve document: "+uri)
    elif isinstance(nodeSet, Types.NodesetType):
        if type(object) == type([]):
            for curr_node in object:
                result = result + Document(
                    context,
                    Conversions.StringValue(curr_node),
                    nodeSet
                    )
        else:
            try:
                uri = sheet._input_source.getUriResolver().normalize(Conversions.StringValue(object),baseUri)
                if context.documents.has_key(uri):
                    result = [context.documents[uri]]
                else:
                    new_stream = context.processor._documentInputSource._openStream(uri)
                    isrc = context.processor._documentInputSource._clone(new_stream,uri)
                    doc = context.processor._docReader.parse(isrc)
                    context.addDocument(doc, uri)
                    result = [doc]
            except IOError:
                context.processor.warning("Unable to retrieve document: "+uri)
    return result


def Key(context, qname, keyList):
    qname = Conversions.StringValue(qname)
    if not qname:
        raise XsltRuntimeException(Error.INVALID_QNAME_ARGUMENT,
                                   context.currentInstruction, qname)
    doc = context.node.ownerDocument or context.node
    try:
        context_key = context.processor.keys[doc]
    except KeyError:
        sheet = context.processor.stylesheet
        sheet.updateKeys(doc, context.processor)
        context_key = context.processor.keys[doc]

    result = []
    #FIXME: actual test should ensure split_name is a QName
    split_name = context.expandQName(Conversions.StringValue(qname))
    if context_key.has_key(split_name):
        a_dict = context_key[split_name]
        if not isinstance(keyList, Types.NodesetType):
            keyList = [keyList]
        for key in keyList:
            key = Conversions.StringValue(key)
            result.extend(a_dict.get(key, []))
    return result


def Current(context):
    return [context.currentNode]


def UnparsedEntityUri(context, name):
    if hasattr(context.node.ownerDocument, '_unparsedEntities'):
        return context.node.ownerDocument._unparsedEntities.get(name, EMPTY_NAMESPACE)
    return ''


def GenerateId(context, nodeSet=None):
    if nodeSet is not None and type(nodeSet) != type([]):
        raise XsltRuntimeException(Error.WRONG_ARGUMENT_TYPE, context.currentInstruction)
    if not nodeSet:
        return u'id' + `id(context.node)`
    else:
        node = Util.SortDocOrder(context, nodeSet)[0]
        return u'id' + `id(node)`


def SystemProperty(context, qname):
    qname = Conversions.StringValue(qname)
    #FIXME: actual test should ensure split_name is a QName
    if not qname:
        raise XsltRuntimeException(Error.INVALID_QNAME_ARGUMENT,
                                   context.currentInstruction, qname)
    (uri, local) = context.expandQName(qname)
    if uri == XSL_NAMESPACE:
        if local == 'version':
            return 1.0
        if local == 'vendor':
            return u"Fourthought Inc."
        if local == 'vendor-url':
            return u"http://4Suite.org"
    elif uri == 'http://xmlns.4suite.org/xslt/env-system-property':
        return unicode(os.environ.get(local, ''))
    elif uri == 'http://xmlns.4suite.org':
        if local == 'version':
            return __version__
    return u''


def FunctionAvailable(context, qname):
    qname = Conversions.StringValue(qname)
    #FIXME: actual test should ensure split_name is a QName
    if not qname:
        return boolean.false
    split_name = context.expandQName(qname)
    if context.functions.has_key(split_name):
        return boolean.true
    else:
        return boolean.false


def ElementAvailable(context, qname):
    qname = Conversions.StringValue(qname)
    #FIXME: actual test should ensure split_name is a QName
    if not qname:
        return boolean.false
    split_name = context.expandQName(qname)

    if split_name[0] == EMPTY_NAMESPACE:
        return boolean.false
    elif split_name[0] == XSL_NAMESPACE:
        if _ELEMENT_MAPPING.has_key(split_name[0]):
            return boolean.true
        else:
            return boolean.false
    elif context.processor.extElements.has_key(split_name):
        return boolean.true
    else:
        return boolean.false


def FormatNumber(context, number, formatString, decimalFormatName=None):
    num = Conversions.NumberValue(number)

    format_string = Conversions.StringValue(formatString)

    format_name = context.expandQName(decimalFormatName)
    decimal_format = context.processor.stylesheet.decimalFormats[format_name]

    return routines.FormatNumber(num, format_string, decimal_format)


CoreFunctions = {
    (EMPTY_NAMESPACE, 'document'): Document,
    (EMPTY_NAMESPACE, 'key'): Key,
    (EMPTY_NAMESPACE, 'current'): Current,
    (EMPTY_NAMESPACE, 'generate-id'): GenerateId,
    (EMPTY_NAMESPACE, 'system-property'): SystemProperty,
    (EMPTY_NAMESPACE, 'function-available'): FunctionAvailable,
    (EMPTY_NAMESPACE, 'element-available'): ElementAvailable,
    (EMPTY_NAMESPACE, 'format-number'): FormatNumber,
    (EMPTY_NAMESPACE, 'unparsed-entity-uri'): UnparsedEntityUri
    }

